module PictureHandler
  module Exceptions

    class CustomMessage < StandardError
      alias :orig_to_s :to_s
      def to_s
        @data
      end

      def message
        @data
      end

    end

    class CustomError < CustomMessage
      def initialize(error_description)
        @data = "#{error_description}"
      end
    end

    class VersionError < CustomMessage
      def initialize(version_name)
        @data = "The Version: \"#{version_name}\" is use as a FROM but doesn't exist."
      end
    end

    class InitError < CustomMessage
      def initialize(uploader_name)
        @data = "The Uploader: \"#{uploader_name}\" have not been initialized."
      end
    end

    class WrongArgument < CustomMessage
      def initialize(method)
        @data = "The argument(s) you provided for the method: \"#{method}\" are incomplete or wrong."
      end
    end

    class NoMethodError < CustomMessage
      def initialize(method)
        @data = "The minimagick method: \"#{method}\" is not a minimagick supported method."
      end
    end

    class OpenError < CustomMessage
      def initialize(source)
        @data = "The image at source: \"#{source}\" could not be open."
      end
    end

    class WriteError < CustomMessage
      def initialize(path)
        @data = "The image could not be write at \"#{path}\" path."
      end
    end

  end
end


