module PictureHandler

  module Mounter

    def self.included(base)

      base.extend		ClassMethods
      base.class_eval do
        cattr_accessor :column
        cattr_reader :uploaders

        attr_accessor :remote_path
        attr_reader :model_filename, :uploader, :old_record, :populated
      end
      base.class_variable_set('@@uploaders', {})

    end


    module ClassMethods

      # Choose the field to action the image with
      def storage_field(column, picture_handler)

        before_save :picture_store
        after_save :picture_delete_old

        before_destroy :picture_delete

        # Save the COLUMN !
        self.column = column

        # Put this uploader in the uploader list
        self.uploaders[column] = picture_handler

        # Inside this class_eval, we are in the instance so you can access Model.picture
        class_eval <<-RUBY

          # Overwrite the field 'storage field' so you can return a formated object instead
          def #{column}
            get_object_or_string(self[__method__.to_sym], __method__)
          end

          def #{column}_versions
            self.send(self.column.to_s).versions_info
          end

        RUBY

      end

    end # ClassMethod

    private  # -------- #

    # Every time model.picture is called
    def get_object_or_string(attribute, column)

      build_picture_handler_object(attribute, column) if !@uploader
      @uploader

    end

    # CREATE an instance of PictureUploader::Base if the Object has not been created yet
    def build_picture_handler_object(attribute, column)

      @populated = false
      @column = self.class.column.to_s
      attribute = self.class.uploaders[column].new(self)
      @uploader = attribute
      @model_filename = ""

      @old_record = !self.id.blank?
      # If the record already exist, populate the name
      if @old_record
        # Active Record, write as attribute for Model.#{column}
        @model_filename = read_attribute(@column)
      end

    end


    def register_active_record_attribute

      # If there is no image, it means it is an old record that haven't changed yet (not populated yet)
      raise PictureHandler::Exceptions::CustomError.new(
                "Trying to save ActiveRecord attribute, but there is no image populated yet.") if !@populated || @uploader.image.nil?

      # This return either the filename chosen by the dev or the default filename of PictureUploader::Base class method filename
      @model_filename = @uploader.real_filename + "." + @uploader.image.type.downcase

      # Active Record, write as attribute for Model.#{column}
      write_attribute(@column, @model_filename)
      @model_filename

    end

    # Trigger when ActiveRecord Model.destroy
    def picture_delete

      # Call Model.column field
      column = self.class.column.to_s
      self.send(column) if @uploader.nil?

      @uploader.delete_all

    end

    # Trigger when ActiveRecord before Model.save
    def picture_store

      # Call Model.column field
      self.send(self.class.column.to_s) if @uploader.nil?

      # If we are in an already existing record a no remote_path have been given prior to save
      return if @old_record && self.remote_path.blank?

      # STORE IT
      @uploader.store
      @populated = true

      # Ask for a write inside the ActiveRecord Attribute field
      register_active_record_attribute

    end

    # Trigger when ActiveRecord after Model.save
    def picture_delete_old

      puts "AFTER_SAVE SUPPRESSION DES FICHIERS DEJA UP !!"


    end

  end

end