module PictureHandler
  module Uploader
    module ImgVersion
      extend ActiveSupport::Concern

      def version(sym, options={}, &block)
        # If the version already exist, the new one will overwrite the old one
        p "You already have a version named \"#{sym.to_s}\". Old one is being replaced" if @versions[sym] && @versions[sym][:block]

        # Fill the versions[:version_name][:block] with the block to be executed later
        @versions[sym] = { block: block } if block_given?

        # Fill the versions[:version_name][:from_version] with the symbol of the version from witch it should inherit
        @versions[sym].merge!( {from_version: options[:from_version]} )
      end

      # The process is the function called inside the versions declared by the user
      def process(args)
        image = @versions[@current_version][:image]

        args.each do |key, value|
          case key
            when :resize_to_fill
              self.class.resize_to_fill(image, value)
            when :quality
              self.class.quality(image, value)
            # when :rounded
            # self.class.rounded(image, value)
            else
              raise PictureHandler::Exceptions::NoMethodError.new(key)
          end
        end
      end

      module ClassMethods

        # -------------------------------------------------------------- #
        # ------------- Method accepted into a version  ---------------- #
        # -------------------------------------------------------------- #

        def resize_to_fill(image, value)
          width = value.first
          height = value.last
          gravity = 'Center'
          raise PictureHandler::Exceptions::WrongArgument.new(key),
                "You need to specify a width and height as argument of 'resize_to_fill' method." if width.blank? || height.blank?

          cols, rows = image[:dimensions]
          image.combine_options do |cmd|
            if width != cols || height != rows
              scale_x = width/cols.to_f
              scale_y = height/rows.to_f
              if scale_x >= scale_y
                cols = (scale_x * (cols + 0.5)).round
                rows = (scale_x * (rows + 0.5)).round
                cmd.resize "#{cols}"
              else
                cols = (scale_y * (cols + 0.5)).round
                rows = (scale_y * (rows + 0.5)).round
                cmd.resize "x#{rows}"
              end
            end
            cmd.gravity gravity
            cmd.background "rgba(255,255,255,0.0)"
            cmd.extent "#{width}x#{height}" if cols != width || rows != height
          end
          # image = yield(image) if block_given?
        end

        def quality(image, value)
          image.quality(value)
        end

      end # ClassMethod

    end
  end
end