module PictureHandler
  module Uploader
    module MiniMagick

      def self.included(base)
        base.extend		ClassMethods
        base.class_eval do
          begin
            require "mini_magick"
          rescue LoadError => e
            e.message << " (You may need to install the mini_magick gem)"
            raise e
          end
        end
      end

      # Common method for all file_system
      module ClassMethods

        # Open the source File/path/ or URL, raise an error if blank
        def open_source(source_path)
          raise PictureHandler::Exceptions::OpenError.new("nil") if source_path.blank?

          image = ::MiniMagick::Image.open(source_path)
          if image.blank?
            raise PictureHandler::Exceptions::OpenError.new(source_path)
          end
          image
        end

        def create_directories_for_path(dir_tokens)
          1.upto(dir_tokens.size) do |n|
            dir = dir_tokens[0...n]
            dir_str = dir.join("/")
            Dir.mkdir(dir_str) unless Dir.exist?(dir_str)
          end
        end

        # Works for Windows C:\Files\etc or Linux /usr/app/etc
        def split_dirname_path(some_path)
          dirname = File.dirname(some_path)
          tokens = dirname.split(/[\/\\]/)
          tokens
        end

      end # ClassMethods

    end
  end
end